#pragma once
#include <cstdlib>

//Copyright (C) cocosimulator.org 2023
//generated by AmsterCHEM's COM CAPE-OPEN Wizard

//a collection of string conversion classes. Can be used as if they are a function.

#define STRINGCONVERSION_STACKVARIABLELENGTH 128

class MultibyteToUnicode {
	OLECHAR stack[STRINGCONVERSION_STACKVARIABLELENGTH];
	OLECHAR *target;

	public:

	MultibyteToUnicode(const char *str) {
		size_t len=strlen(str);
		if (len<STRINGCONVERSION_STACKVARIABLELENGTH) {
			//use stack
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			mbstowcs(stack,str,STRINGCONVERSION_STACKVARIABLELENGTH);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
			target=stack;
		} else {
			//use heap
			len++;
			target=new OLECHAR[len];
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			mbstowcs(target,str,len);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
		}
	}

	MultibyteToUnicode(const std::string &str) {
		if (str.size()<STRINGCONVERSION_STACKVARIABLELENGTH) {
			//use stack
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			mbstowcs(stack,str.c_str(),STRINGCONVERSION_STACKVARIABLELENGTH);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
			target=stack;
		} else {
			//use heap
			size_t len=str.size();
			len++;
			target=new OLECHAR[len];
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			mbstowcs(target,str.c_str(),len);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
		}
	}

	~MultibyteToUnicode() {
		if (stack!=target) delete []target;
	}

	operator const OLECHAR *() {return target;}

};

class UnicodeToMultibyte{
	char stack[STRINGCONVERSION_STACKVARIABLELENGTH];
	char *target;

	public:

	UnicodeToMultibyte(const std::wstring &str) {
		if (str.size()<(STRINGCONVERSION_STACKVARIABLELENGTH>>1)) {
			//use stack, the result will fit 
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			wcstombs(stack,str.c_str(),STRINGCONVERSION_STACKVARIABLELENGTH);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
			target=stack;
		} else {
			//use heap, the result may not fit
			size_t len=str.size();
			len<<=1; 
			len++;
			target=new char[len];
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			wcstombs(target,str.c_str(),len);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
		}
	}

	UnicodeToMultibyte(const OLECHAR *str) {
		size_t len=wcslen(str);
		if (len<(STRINGCONVERSION_STACKVARIABLELENGTH>>1)) {
			//use stack, the result will fit 
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			wcstombs(stack,str,STRINGCONVERSION_STACKVARIABLELENGTH);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
			target=stack;
		} else {
			//use heap, the result may not fit
			len<<=1; 
			len++;
			target=new char[len];
			#ifdef _MSC_VER
			#pragma warning( push ) 
			#pragma warning( disable : 4996) 
			#endif
			wcstombs(target,str,len);
			#ifdef _MSC_VER
			#pragma warning( pop ) 
			#endif
		}
	}

	~UnicodeToMultibyte() {
		if (target!=stack) delete []target;
	}

	operator const char *() {return target;}
};

//these are for out-of-place conversion. For in-place conversion
// such as inside a wstring, use std::transform instead

template <int Convert(int c)> class StringConversion {
	
	OLECHAR stack[STRINGCONVERSION_STACKVARIABLELENGTH];
	OLECHAR *target;

	public:

	StringConversion(const OLECHAR *str) {
		const OLECHAR *source=str;
		const OLECHAR *end=str+STRINGCONVERSION_STACKVARIABLELENGTH-1;
		OLECHAR *tgt;
		tgt=target=stack;
		while (*source) {
			if (source==end) {
				//won't fit, use heap
				int len=(int)wcslen(source)+STRINGCONVERSION_STACKVARIABLELENGTH;
				target=new OLECHAR[len];
				memcpy(target,stack,(STRINGCONVERSION_STACKVARIABLELENGTH-1)<<1);
				tgt=target+STRINGCONVERSION_STACKVARIABLELENGTH-1;
				while (*(source)) *(tgt++)=Convert(*(source++));
				break;
			}
			*(tgt++)=Convert(*(source++));
		}
		*tgt=0;
	}

	StringConversion(const std::wstring &str) {
		if (str.size()>=STRINGCONVERSION_STACKVARIABLELENGTH) {
			target=new OLECHAR[str.size()+1];
		} else {
			target=stack;
		}
		OLECHAR *tgt=target;
		const OLECHAR *source=str.c_str();
		while (*source) *(tgt++)=Convert(*(source++));
		*tgt=0;
	}

	~StringConversion() {
		if (target!=stack) delete []target;
	}

	operator const OLECHAR *() {return target;}

};

using MakeLowerCase=StringConversion<std::tolower>;
using MakeUpperCase=StringConversion<std::toupper>;

#undef STRINGCONVERSION_STACKVARIABLELENGTH